@@ -1,7 +1,7 @@
 package csv
 
 import (
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
+	"github.com/operator-framework/api/pkg/operators/v1alpha1"
 )
 
 // WatchNotification is an sink interface that can be used to get notification
